/*
 * bg_threads_helper.c
 *
 * Copyright (C) 2019 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <bg.h>

#if defined (BG_PARAM_THREADS) // [
int bg_threads_helper_create(bg_threads_helper_t *helper)
{
#if defined (_WIN32) // [
  ///////////////////////////////////////////////////////////////////////////
  helper->hEvent=CreateEventW(
    NULL,     // LPSECURITY_ATTRIBUTES lpEventAttributes,
    FALSE,    // BOOL                  bManualReset,
    FALSE,    // BOOL                  bInitialState,
    NULL      // LPCWSTR               lpName
  );

  if (!helper->hEvent) {
    DVMESSAGE("creating event (%lu)",GetLastError());
    goto e_event;
  }
#else // ] [
  ///////////////////////////////////////////////////////////////////////////
  if (pthread_mutex_init(&helper->mutex,NULL)<0) {
    DVMESSAGE("creating mutex (%d)",errno);
    goto e_mutex;
  }

  ///////////////////////////////////////////////////////////////////////////
  if (pthread_cond_init(&helper->cond,NULL)<0) {
    DVMESSAGE("creating cond (%d)",errno);
    goto e_cond;
  }
#endif // ]

  /////////////////////////////////////////////////////////////////////////////
  return 0;
//cleanup:
#if defined (_WIN32) // [
  CloseHandle(helper->hEvent);
e_event: ;
#else // ] [
  pthread_cond_destroy(&helper->cond);
e_cond:
  pthread_mutex_destroy(&helper->mutex);
e_mutex: ;
#endif // ]
  return -1;
}

void bg_threads_helper_destroy(bg_threads_helper_t *helper)
{
#if defined (_WIN32) // [
  CloseHandle(helper->hEvent);
#else // ] [
  pthread_cond_destroy(&helper->cond);
  pthread_mutex_destroy(&helper->mutex);
#endif // ]
}
#endif // ]
